<?php

namespace App\Model;

use Nette;
use Nette\Database\Table\ActiveRow;

/**
 * Class for handling activities related to defences
 * @package App\Model
 */
class DefenceManager {
    use Nette\SmartObject;

    private Nette\Database\Explorer $database;
    private TeamManager $teamManager;

    public function __construct(Nette\Database\Explorer $database,
                                TeamManager $teamManager) {
        $this->database = $database;
        $this->teamManager = $teamManager;
    }

    /**
     * Function returns defences related to given project
     * @param ActiveRow|null $project
     * @return Nette\Database\Table\GroupedSelection|null
     */
    public function getDefencesRelToProject(ActiveRow|null $project) {
        if(is_null($project)) {
            return null;
        }

        return $project->related('DEFENCE');
    }

    /**
     * Function creates new defence record
     * @param $teamId
     * @param $termNumber
     * @param $termDate
     * @param $termResult
     */
    public function createDefence($teamId, $termNumber, $termDate, $termResult) {
        $projectId = $this->teamManager->getTeamById($teamId)->related('PROJECT')->fetch();
        $this->database->table('DEFENCE')->insert([
            'project' => $projectId,
            'attempt' => $termNumber,
            'datetime' => $termDate,
            'result' => $termResult,
        ]);
    }

    /**
     * Function to remove defence record
     * @param $defenceId
     */
    public function removeDefence($defenceId) {
        $this->database->table('DEFENCE')->get($defenceId)->delete();
    }

}