<?php

namespace App\Presenters;

use App\Model\ConfigManager;
use App\Model\RoleManager;
use App\Model\StudentManager;
use App\Model\TeamManager;
use Nette;
use Contributte;
use Contributte\Monolog\LoggerManager;
use Nette\Http\IResponse;
use Psr\Log\LoggerInterface;
use App\Model\Constants;

/**
 * Presenter class with basic functions and necessary services
 */
class BasePresenter extends Nette\Application\UI\Presenter {

    /** @var LoggerManager **/
    protected LoggerManager $loggerManager;

    /** @var ConfigManager **/
    protected ConfigManager $configManager;

    /** @var TeamManager */
    protected TeamManager $teamManager;

    /** @var StudentManager */
    protected StudentManager $studentManager;

    /** @var Nette\Database\Explorer*/
    protected Nette\Database\Explorer $database;

    /** @var Nette\Localization\ITranslator @inject */
    public $translator;

    /** @var Contributte\Translation\LocalesResolvers\Session @inject */
    public $translatorSessionResolver;

    public function __construct() {
        parent::__construct();
    }

    public function startup() {
        parent::startup();
        /*
        if(!$this->getUser()->isLoggedIn()) {
            $this->redirect("Sign:in");
        }
        */

        if(!$this->canAccessPage()) {
            //$this->redirect("Sign:in");
            $this->error("Nejste oprávněni přistoupit k těmto informacím.", IResponse::S403_FORBIDDEN);
        }

        if($this->getUser()->isInRole(RoleManager::$ROLE_LEADER)) {
            $this->template->hasAssignedAssignment = $this->teamManager->hasTeamAssignmentAR($this->studentManager
                ->teamRelToStudent($this->studentManager->getStudentByUser($this->getUser()->getId())));
        }

    }

    public function beforeRender() {
        parent::beforeRender(); // TODO: Change the autogenerated stub
        $this->template->config = $this->configManager->getConfig();
    }

    /**
     * DI injection of Logger manager
     * @param LoggerManager $loggerManager
     */
    public function injectLoggerManager(LoggerManager $loggerManager): void {
        $this->loggerManager = $loggerManager;
    }

    public function injectConfigManager(ConfigManager $configManager): void {
        $this->configManager = $configManager;
    }

    public function injectTeamManager(TeamManager $teamManager) {
        $this->teamManager = $teamManager;
    }

    public function injectStudentManager(StudentManager $studentManager) {
        $this->studentManager = $studentManager;
    }
    public function injectDatabase(Nette\Database\Explorer $database): void {
        $this->database = $database;
    }


    public function handleChangeLocale(string $locale) {
        $this->translatorSessionResolver->setLocale($locale);
        $this->loggerManager->get('default')->info("Locale is changed",["user_id"=>$this->getUser()->getId(),"locale"=>$locale]);
    }

    public function handleResetDatabase() {

        if($this->configManager->getConfig()['testing']['enabled']) {
            Nette\Database\Helpers::loadFromFile($this->database->getConnection(), $this->configManager->getConfig()['testing']['databaseResetFileName']);
            $this->flashMessage("Úspěch;Databáze byla uvedena do požadovaného stavu.", "bg-success");
            $this->redirect('this');
        }

    }
    
    public function canAccessPage():bool {
        return true;
    }


}