<?php

declare(strict_types=1);

namespace App\Presenters;

use App\Model\MeetingFacade;
use App\Model\ProjectManager;
use App\Model\RoleManager;
use Nette;

/**
 * Presenter class to handle requests to page with mentored projects
 */
class MentoredProjectsPresenter extends BasePresenter {

    private ProjectManager $projectManager;
    private MeetingFacade $meetingFacade;

    public Nette\Database\Explorer $database;

    public function __construct(ProjectManager $projectManager, MeetingFacade $meetingFacade) {
        parent::__construct();

        $this->projectManager = $projectManager;
        $this->meetingFacade = $meetingFacade;
    }

    public function renderDefault(int $id = -1) {

        if($id == -1) {
            $id = $this->getUser()->getId();
        }

        $this->template->mentor = $this->database->table('USER')->get($id);

        $this->template->mentoredProjects = $this->projectManager
            ->getMentoredProjects($id);

        $this->template->meetings = $this->meetingFacade->getMeetingsLinkedWith('mentor', $id);
    }

    public function canAccessPage(): bool {
        if(!$this->user->isLoggedIn()) {
            return false;
        } else if($this->user->isInRole(RoleManager::$ROLE_MENTOR) || $this->user->isInRole(RoleManager::$ROLE_GARANT)) {
            return true;
        }
        return false;
    }
}