<?php

namespace App\Presenters;

use App\Components\PasswordChangeForm;
use App\Model\RoleManager;
use App\Model\UserManager;
use Nette\Application\UI\Form;
use Nette\Utils\ArrayHash;
use App\Components\ContactEditForm;

/**
 * Presenter class to handle requests to page with profile editation
 */
class ProfileEditPresenter extends BasePresenter {

    protected UserManager $userManager;
    protected PasswordChangeForm $passwordChangeForm;
    protected ContactEditForm $contactEditForm;

    public function __construct(UserManager $userManager,
                                PasswordChangeForm $passwordChangeForm,
                                ContactEditForm $contactEditForm) {
        $this->userManager = $userManager;
        $this->passwordChangeForm = $passwordChangeForm;
        $this->contactEditForm = $contactEditForm;
    }

    public function createComponentPasswordChangeForm():Form {

        $form = $this->passwordChangeForm->create();

        $form->onSuccess[] = [$this, 'changePassword'];

        return $form;
    }

    public function changePassword(Form $form, ArrayHash $values) {

        if($this->userManager->changePasswordIfPasswordValid($this->getUser()->getId(), $values['oldPassword'], $values['newPassword1'])) {
            $this->loggerManager->get('default')->info('The user has changed the password', [
                'user_identity' => $this->getUser()->getIdentity()->getData(),
            ]);
            $this->flashMessage('Úspěch;Změna hesla proběhla úspěšně.', 'bg-success');
            $this->redirect('this');
        } else {
            $this->flashMessage('Chyba;Staré heslo se neshoduje s našimi záznamy.', 'bg-danger');
        }
    }

    public function createComponentContactEditForm():Form {

        $form = $this->contactEditForm->create($this->getUser()->getId());

        $form->onSuccess[] = [$this, 'updateContact'];

        return $form;
    }

    public function updateContact(Form $form, ArrayHash $values) {

        if($this->getUser()->isInRole(RoleManager::$ROLE_MENTOR) || $this->getUser()->isInRole(RoleManager::$ROLE_GARANT)) {
            $contact = $this->userManager->getContact($this->userManager->getUser($this->getUser()->getId()));
            $this->userManager->updateContact($contact, $values['firstname'], $values['lastname'], $values['email'], $values['phone'], $values['info']);

            $this->loggerManager->get('default')->info('The user has changed their contact details', [
                'user_identity' => $this->getUser()->getIdentity()->getData(),
            ]);

            $this->flashMessage('Úspěch;Kontaktní informace byly úspěšně upraveny.', 'bg-success');
            return;
        }


    }

    public function canAccessPage(): bool {
        if(!$this->getUser()->isLoggedIn()) {
            return false;
        } else {
            return true;
        }
    }

}