<?php

namespace App\Presenters;

use App\Components\MentorAddForm;
use App\Model\RoleManager;
use App\Model\UserManager;
use Nette\Application\UI\Form;

/**
 * Presenter class to handle requests to page with user management
 */
class UserManagementPresenter extends BasePresenter {

    protected UserManager $userManager;
    protected MentorAddForm $mentorAddForm;

    public function __construct(UserManager $userManager,
                                MentorAddForm $mentorAddForm) {
        $this->userManager = $userManager;
        $this->mentorAddForm = $mentorAddForm;
    }

    public function renderDefault() {
        $this->template->mentors = $this->userManager->getAllMentors()->where('can_login', 1);
        $this->template->leaders = $this->userManager->getAllLeaders()->where('can_login', 1);
    }

    public function handleMentorRemove(int $mentorId) {
        $this->userManager->disableLoginUser($mentorId);
        $this->loggerManager->get('default')->info('The user removed access to the mentor', [
            'user_identity' => $this->getUser()->getIdentity()->getData(),
            'leader_id' => $mentorId,
        ]);
        $this->flashMessage('Úspěch;Požadavek na odstranění mentora byl zpracován.', 'bg-success');
        $this->redirect('this');
    }

    public function handleLeaderRemove(int $leaderId) {

        $this->userManager->disableLoginUser($leaderId);
        $this->loggerManager->get('default')->info('The user removed access to the leader', [
            'user_identity' => $this->getUser()->getIdentity()->getData(),
            'leader_id' => $leaderId,
        ]);
        $this->flashMessage('Úspěch;Požadavek na odstranění vedoucího byl zpracován.', 'bg-success');
        $this->redirect('this');

    }

    public function createComponentMentorAddForm():Form {
        $form = $this->mentorAddForm->create();

        $form->onSuccess[] = function() {
            $this->loggerManager->get('default')->info('User created new mentor user', [
                'user_identity' => $this->getUser()->getIdentity()->getData(),
            ]);
            $this->flashMessage('Úspěch;Nový mentor byl vytvořen.', 'bg-success');
            $this->redirect('this');
        };

        return $form;
    }

    public function canAccessPage(): bool {
        if(!$this->user->isLoggedIn()) {
            return false;
        } else if($this->user->isInRole(RoleManager::$ROLE_GARANT)) {
            return true;
        }
        return false;
    }
}