<?php

namespace App\Components;

use App\Model\UserManager;
use Grido\Components\Component;
use Nette\Application\UI\Form;
use Nette\Utils\ArrayHash;

/**
 * Form component class form password change
 * @package App\Components
 */
class PasswordChangeForm {

    protected UserManager $userManager;

    /**
     * Component constructor
     * @param UserManager $userManager
     */
    public function __construct(UserManager $userManager) {
        $this->userManager = $userManager;
    }

    /**
     * Factory function for password change form
     * @return Form
     */
    public function create():Form {

        $form = new Form();

        $form->addPassword('oldPassword', 'Staré heslo')->setRequired('Položka musí být zadána')
        ->setHtmlAttribute('class', 'form-control');
        $form->addPassword('newPassword1', 'Nové heslo')->setRequired('Položka musí být zadána')
            ->addRule(Form::MIN_LENGTH, "Heslo musí být dlouhé alespoň %d znaků.", 8)
            ->addRule(Form::PATTERN, 'Zadaný řetězec obsahuje nepovolené znaky.', '^[a-zA-Z0-9#?!@$%^&*-]*$')
            ->addRule(Form::PATTERN, 'Heslo musí obsahovat alespoň jedno malé písmeno.', '^[a-zA-Z0-9#?!@$%^&*-]*[a-z]+[a-zA-Z0-9#?!@$%^&*-]*$')
            ->addRule(Form::PATTERN, 'Heslo musí obsahovat alespoň jedno velké písmeno.', '^[a-zA-Z0-9#?!@$%^&*-]*[A-Z]+[a-zA-Z0-9#?!@$%^&*-]*$')
            ->addRule(Form::PATTERN, 'Heslo musí obsahovat alespoň jedna číslice.', '^[a-zA-Z0-9#?!@$%^&*-]*[0-9]+[a-zA-Z0-9#?!@$%^&*-]*$')
            ->setHtmlAttribute('class', 'form-control');
        $form->addPassword('newPassword2', 'Nové heslo znovu')->setRequired('Položka musí být zadána')
            ->addRule(Form::EQUAL, "Hesla se musí shodovat", $form['newPassword1'])
            ->setHtmlAttribute('class', 'form-control');

        $form->addSubmit('send', 'Změnit heslo');

        return $form;
    }

}