<?php

namespace App\Components;

use App\Model\PeriodManager;
use App\Model\StudentManager;
use Nette\Application\UI\Form;
use Nette\Utils\ArrayHash;

/**
 * Form component class for creating new student
 */
class StudentAddForm {

    protected PeriodManager $periodManager;
    protected StudentManager $studentManager;

    /**
     * Component constructor
     * @param PeriodManager $periodManager
     * @param StudentManager $studentManager
     */
    public function __construct(PeriodManager $periodManager,
                                StudentManager $studentManager) {
        $this->periodManager = $periodManager;
        $this->studentManager = $studentManager;
    }

    /**
     * Factory function for creating student creation form
     * @return Form
     */
    public function create():Form {
        $form = new Form();

        $form->addText('firstname', 'Křestní jméno')
            ->setRequired('Křestní jméno musí být zadáno.')
            ->setHtmlAttribute('class', 'form-control');
        $form->addText('lastname', 'Příjmení')
            ->setRequired('Příjmení musí být zadáno.')
            ->setHtmlAttribute('class', 'form-control');
        $form->addText('email', 'Email')
            ->setRequired('Email musí být zadán')
            ->setHtmlAttribute('class', 'form-control');
        $form->addText('login', 'Login (do StaGu)')
            ->setRequired('Login musí být zadán')
            ->setHtmlAttribute('class', 'form-control');
        $form->addText('personalID', 'Osobní číslo')
            ->setRequired('Osobní číslo musí být zadáno.')
            ->setHtmlAttribute('class', 'form-control');

        $form->addText('fieldOfStudy', 'Obor studia')
            ->setHtmlAttribute('class', 'form-control');

        $form->addText('faculty', 'Studovaná fakulta')
            ->setHtmlAttribute('class', 'form-control');


        $periodsArray = array();
        $periods = $this->periodManager->getAllPeriods();

        foreach ($periods as $period) {
            $periodsArray[$period->id] = $period->label;
        }

        $form->addSelect('period', 'Ročník TSP', $periodsArray)
            ->setHtmlAttribute('class', 'form-control')->setRequired('Ročník TSP musí být vyplněn.');

        $form->addSubmit('send');

        $form->onSuccess[] = [$this, 'saveForm'];

        return $form;
    }

    /**
     * Fuctions that is triggered by a successful form submission
     * @param Form $form
     * @param ArrayHash $values
     */
    public function saveForm(Form $form, ArrayHash $values) {
        $this->studentManager->createStudent($values['login'], $values['firstname'], $values['lastname'],
            $values['personalID'], $values['email'], $values['fieldOfStudy'], $values['faculty'], $values['period']);
    }

}