<?php

namespace App\Components;

use App\Model\PeriodManager;
use App\Model\StudentManager;
use App\Model\TeamRoleManager;
use Nette\Application\UI\Form;
use Nette\Utils\ArrayHash;

/**
 * Form component class for importing list of students
 */
class StudentImportForm extends Form {

    protected StudentManager $studentManager;
    protected PeriodManager $periodManager;

    /**
     * Component constructor
     * @param StudentManager $studentManager
     * @param PeriodManager $periodManager
     */
    public function __construct(StudentManager $studentManager,
                                PeriodManager $periodManager) {
        parent::__construct();

        $this->studentManager = $studentManager;
        $this->periodManager = $periodManager;
    }

    /**
     * Factory function for creating student import form
     * @return Form
     */
    public function create() {

        $form = new Form();

        $form->addUpload('csvFile', 'Soubor se studenty')->setRequired('Soubor se zadáním je povinný.');

        $periodsArray = array();
        $periods = $this->periodManager->getAllPeriods();

        foreach ($periods as $period) {
            $periodsArray[$period->id] = $period->label;
        }

        $form->addSelect('period', 'Ročník TSP', $periodsArray)
            ->setHtmlAttribute('class', 'form-control');

        $form->addSubmit('send');

        $form->onSuccess[] = [$this, 'saveForm'];

        return $form;
    }

    /**
     * Function that is triggered by a successful form submission
     * @param Form $form
     * @param ArrayHash $values
     */
    public function saveForm(Form $form, ArrayHash $values) {
        $this->studentManager->importStudents($values['period'], $values['csvFile']);
    }
}