<?php

namespace App\Components;

use App\Model\StudentManager;
use App\Model\TeamManager;
use App\Model\TagManager;
use App\Model\UserManager;
use http\Exception;
use Nette\Application\AbortException;
use Nette\Application\UI\Form;
use Nette\Database\Explorer;
use Nette\Security\AuthenticationException;
use Nette\Security\User;
use Nette\Utils\ArrayHash;

/**
 * Form component class for edit team information
 * @package App\Components
 */
class TeamEditForm extends Form {
    protected TeamManager $teamManager;
    protected StudentManager $studentManager;

    /**
     * Component constructor
     * @param TeamManager $teamManager
     * @param StudentManager $studentManager
     */
    public function __construct(TeamManager $teamManager, StudentManager $studentManager) {
        parent::__construct();
        $this->teamManager = $teamManager;
        $this->studentManager = $studentManager;
    }

    /**
     * Factory function for creating team edit form
     * @param int $teamId
     * @return Form
     */
    public function create(int $teamId = -1) {
        //$values = $this->tagManager->getAllTags();
        $defaults = [];
        $form = new Form();
        if ($teamId >= 0) {
            $defaults = $this->teamManager->getTeamAdvances($teamId);
        }
        $team = $this->teamManager->getTeam($teamId);
        $teamName = $team["name"];
        $teamLeader = $this->teamManager->getTeamLeader($teamId);
        $leaderName = $teamLeader["firstname"] . " " . $teamLeader["lastname"]." (".$teamLeader["personalID"].")";
        $teamMembers = $this->teamManager->getTeamMembersExceptLeader($teamLeader["id"],$teamId);
        $studentsWithoutTeam = $this->studentManager->getStudentsWithoutTeam();
        $studentsAvailable = $teamMembers+ $studentsWithoutTeam;
        $form->addText("name","Název týmu")
        ->setHtmlAttribute('class="form-control"')
        ->setDefaultValue($teamName)
        ->setRequired(true);


        $form->addText("teamLeaderName","Jméno vedoucího týmu (Osobní číslo)")
            ->setHtmlAttribute('class="form-control"')
            ->setHtmlAttribute("readonly")
            ->setDefaultValue($leaderName);


        $form->addMultiSelect("members", "Členové týmu", $studentsAvailable)
            ->setHtmlAttribute('class="selectpicker form-control" multiple data-live-search="true" title="Vyberte prosím člena týmu"')
            ->setHtmlAttribute("multiple=multiple")
            ->setHtmlAttribute("style= width:100%")->setDefaultValue(array_flip($teamMembers));

        $form->addSelect("complete","Kompletní tým",["Ne","Ano"])
        ->setHtmlAttribute('class="form-control"')
        ->setDefaultValue($team['is_complete']);

        $form->addHidden("teamId",$teamId);
        $form->addSubmit('save', 'Potvrdit změny');
        $form->addButton('cancel', 'Zrušit');
        //$form->onValidate[] = [$this, 'validateForm'];
        $form->onSuccess[] = [$this, 'saveForm'];

        return $form;
    }

    /**
     * Function that is triggered by a successful form submission
     * @param Form $form
     * @param ArrayHash $values
     */
    public function saveForm(Form $form, ArrayHash $values) {
        try {
            //TODO
            $this->teamManager->updateTeam($values['teamId'], $values['name'],$values['members']);
            $this->teamManager->changeTeamState($values['teamId'],$values["complete"]);

        }  catch (\Exception $e) {

        }
    }
}