<?php
/**
 * TEST: Test for get valid name from args
 * @testCase
 */

use App\Model\Utils;
use Tester\Assert;

require __DIR__ . '/bootstrap.php';

class GetArrayForDynamicInsertTest extends Tester\TestCase
{


    /**
     * @test
     */
    public function testArrayClassic_1()
    {
        $id = 2;
        $subIds = [1, 2, 3, 4];
        $dbIdName = "id";
        $dbRelItName = "item";

        $expected = [[$dbIdName => 2, $dbRelItName => 1],
            [$dbIdName => 2, $dbRelItName => 2],
            [$dbIdName => 2, $dbRelItName => 3],
            [$dbIdName => 2, $dbRelItName => 4]];

        $actual = Utils::getArrayForDynamicInsert($id,$subIds,$dbIdName,$dbRelItName);

        Assert::equal($expected,$actual,"Arrays are not the same");

    }

    /**
     * @test
     */
    public function testArrayClassic_2()
    {
        $id = 80;
        $subIds = [1];
        $dbIdName = "id";
        $dbRelItName = "item";

        $expected = [[$dbIdName => 80, $dbRelItName => 1]];

        $actual = Utils::getArrayForDynamicInsert($id,$subIds,$dbIdName,$dbRelItName);

        Assert::equal($expected,$actual,"Arrays are not the same");

    }

    /**
     * @test
     */
    public function testArrayException_1() {
        $id = -5;
        $subIds = [1];
        $dbIdName = "id";
        $dbRelItName = "item";

        $expected = [[$dbIdName => 80, $dbRelItName => 1]];


        Assert::exception(function () use ($dbRelItName, $dbIdName, $subIds, $id) {
            $actual = Utils::getArrayForDynamicInsert($id,$subIds,$dbIdName,$dbRelItName);
        }, \Nette\InvalidArgumentException::class, "Id must be greater than zero.");
    }

    /**
     * @test
     */
    public function testArrayException_2() {
        $id = 8;
        $subIds = [1];
        $dbIdName = "";
        $dbRelItName = "item";


        Assert::exception(function () use ($dbRelItName, $dbIdName, $subIds, $id) {
            $actual = Utils::getArrayForDynamicInsert($id,$subIds,$dbIdName,$dbRelItName);
        }, \Nette\InvalidArgumentException::class, "dbNameOfId and dbnameOfRelIds must be filled.");
    }

    /**
     * @test
     */
    public function testArrayException_3() {
        $id = 8;
        $subIds = [1];
        $dbIdName = "id";
        $dbRelItName = "";


        Assert::exception(function () use ($dbRelItName, $dbIdName, $subIds, $id) {
            $actual = Utils::getArrayForDynamicInsert($id,$subIds,$dbIdName,$dbRelItName);
        }, \Nette\InvalidArgumentException::class, "dbNameOfId and dbnameOfRelIds must be filled.");
    }

    /**
     * @test
     */
    public function testArrayException_4() {
        $id = -9;
        $subIds = [1];
        $dbIdName = "";
        $dbRelItName = "";


        Assert::exception(function () use ($dbRelItName, $dbIdName, $subIds, $id) {
            $actual = Utils::getArrayForDynamicInsert($id,$subIds,$dbIdName,$dbRelItName);
        }, \Nette\InvalidArgumentException::class, "Id must be greater than zero.");
    }


}

(new GetArrayForDynamicInsertTest())->run();
