<?php
/**
 * TEST: Test actions with other project links
 * @testCase
 */

use App\Model\Utils;
use Tester\Assert;

require __DIR__ . '/bootstrap.php';

class TransformStudentDataToStringTest extends Tester\TestCase
{

	
	/**
     * @test
     */
	public function testStudentNoLeader() {
        $firstName = "John";
        $lastName = "Doe";
        $personalId = "12345AZSC03";
        $leaderString = "";
        $actual = Utils::transformStudentDataToString($firstName,$lastName,$personalId,$leaderString);
        $expected = "John Doe (12345AZSC03) ";
        Assert::equal($expected,$actual,"Student data is not transformed to proper format");
    }

    /**
     * @test
     */
    public function testStudentLeader() {
        $firstName = "Johny";
        $lastName = "Dip";
        $personalId = "12345AZSC03";
        $leaderString = "Mr Leader";
        $actual = Utils::transformStudentDataToString($firstName,$lastName,$personalId,$leaderString);
        $expected = "Johny Dip (12345AZSC03) Mr Leader";
        Assert::equal($expected,$actual,"Student data is not transformed to proper format");
    }

    /**
     * @test
     */
    public function testStudentException_1() {
        $firstName = "";
        $lastName = "Dip";
        $personalId = "12345AZSC03";
        $leaderString = "Mr Leader";
        Assert::exception(function () use ($leaderString, $personalId, $lastName, $firstName) {
            $actual = Utils::transformStudentDataToString($firstName,$lastName,$personalId,$leaderString);
        }, \Nette\InvalidArgumentException::class, "Student must have filled firstName, lastName and personalID");
    }

    /**
     * @test
     */
    public function testStudentException_2() {
        $firstName = "John";
        $lastName = "";
        $personalId = "12345AZSC03";
        $leaderString = "";
        Assert::exception(function () use ($leaderString, $personalId, $lastName, $firstName) {
            $actual = Utils::transformStudentDataToString($firstName,$lastName,$personalId,$leaderString);
        }, \Nette\InvalidArgumentException::class, "Student must have filled firstName, lastName and personalID");
    }

    /**
     * @test
     */
    public function testStudentException_3() {
        $firstName = "John";
        $lastName = "Dip";
        $personalId = "";
        $leaderString = "Mr Leader";
        Assert::exception(function () use ($leaderString, $personalId, $lastName, $firstName) {
            $actual = Utils::transformStudentDataToString($firstName,$lastName,$personalId,$leaderString);
        }, \Nette\InvalidArgumentException::class, "Student must have filled firstName, lastName and personalID");
    }

    /**
     * @test
     */
    public function testStudentException_4() {
        $firstName = "";
        $lastName = "";
        $personalId = "";
        $leaderString = "Mr Leader";
        Assert::exception(function () use ($leaderString, $personalId, $lastName, $firstName) {
            $actual = Utils::transformStudentDataToString($firstName,$lastName,$personalId,$leaderString);
        }, \Nette\InvalidArgumentException::class, "Student must have filled firstName, lastName and personalID");
    }

}

(new TransformStudentDataToStringTest())->run();
